<?php

class PuntuarteHttpClient 
{
	const PUNTUARTE_URL = 'https://www.puntuarte.com';
	const HTTP_REQUEST_TIMEOUT = 3;
	
	public function makePostRequest($url, $data, $timeout = self::HTTP_REQUEST_TIMEOUT, $parse_result = true)
	{		
		$ch = curl_init($url);
		list($is_json, $parsed_data) = PuntuarteHttpClient::jsonOrUrlEncode($data);    
		$content_type = $is_json ? 'application/json' : 'application/x-www-form-urlencoded';                                                                                                                         
		curl_setopt($ch, CURLOPT_CUSTOMREQUEST, 'POST');
		curl_setopt($ch, CURLOPT_POSTFIELDS, $parsed_data);
		curl_setopt($ch, CURLOPT_RETURNTRANSFER, true);
		curl_setopt($ch, CURLOPT_CONNECTTIMEOUT ,$timeout);
		curl_setopt($ch, CURLOPT_HTTPHEADER, array('Content-Type: '.$content_type, 'Content-length: '.Tools::strlen($parsed_data)));                                                                                                                   
		curl_setopt($ch, CURLOPT_SSL_VERIFYHOST, false); /* Added by PrestaShop */
		curl_setopt($ch, CURLOPT_SSL_VERIFYPEER, false); /* Added by PrestaShop */		
		$result = curl_exec($ch);
		curl_close ($ch);	
		return $result;
	}

	private function makeGetRequest($url, $data = array(), $timeout = self::HTTP_REQUEST_TIMEOUT)
	{
		if(count($data) > 0) {
			$url .= '?' . http_build_query($data);	
		}
		$ch = curl_init($url);                                                                                                                     
		curl_setopt($ch, CURLOPT_RETURNTRANSFER, true);
		curl_setopt($ch, CURLOPT_CONNECTTIMEOUT ,$timeout);                                                                                                                   
		curl_setopt($ch, CURLOPT_SSL_VERIFYHOST, false); /* Added by PrestaShop */
		curl_setopt($ch, CURLOPT_SSL_VERIFYPEER, false); /* Added by PrestaShop */		
		$result = curl_exec($ch);
		curl_close ($ch);	
		return $result;
	}
	
	private static function jsonOrUrlEncode($data)
	{
		if (function_exists('json_encode'))
			return array(true, json_encode($data));
		elseif (method_exists('Tools', 'jsonEncode'))
			return array(true, Tools::jsonEncode($data));
		else 
			return array(false, http_build_query($data));
	}
	
	private static function jsonDecode($data, $assoc = false)
	{
		$result = false;
		if (function_exists('json_decode'))
			$result = array(true, json_decode($data, $assoc));
		elseif (method_exists('Tools', 'jsonEncode'))
			$result = array(true, Tools::jsonDecode($data, $assoc));
		else
			$result = array(false);

		if ($result)
		{
			$code = isset($result[1]['status']) ? $result[1]['status']['code'] : $result[1]['code'];
			$message = isset($result[1]['status']) ? $result[1]['status']['message'] : $result[1]['message'];
			$response = isset($result[1]['response']) ? $result[1]['response'] : '';
		    return array('json' => true, 'status_code' => $code, 'status_message' => $message, 'response' => $response);
		}
		else
		{
			$result = preg_match('/code[\W]*(\d*)/', $data, $matches);
			$status_code = $result == 1 ? $matches[1] : '';
			unset($matches, $result);
			$result = preg_match("/message[\W]*[\"'](.*?)[\"']/", $data, $matches);
			$status_message = $result == 1 ? $matches[1] : '';
			unset($matches, $result);
			$result = preg_match('/response[\W]*({)/', $data, $matches, PREG_OFFSET_CAPTURE);
			$response = '';
			if ($result == 1 && isset($matches[1][1]))
				$response = PuntuarteHttpClient::getStringBetweenBrackets(Tools::substr($data, $matches[1][1]));

			return array('json' => false, 'status_code' => $status_code, 'status_message' => $status_message, 'response' => $response);
		}
	}

	private static function getStringBetweenBrackets($data)
	{
		$count = 0;
		if($data[0] != '{')
			return '';
		for ($position = 0; $position < Tools::strlen($data); $position++)
		{
			switch ($data[$position])
			{
				case  '{' :
					$count++;
					break;
				case  '}' :
					$count--;
					break;
					
			}
			if(!$count)
				return Tools::substr($data, 0, $position);	
		}
		return '';
	}
}