<?php

if (!defined('_PS_VERSION_'))
	exit;

class Puntuarte extends Module
{
	private $_html = '';
	private $_httpClient = null;
	private $_puntuarte_module_path = '';	
	private $http_request_file = array('/PuntuarteHttpClient.php'); 
	private $_is_smarty_product_vars_assigned = false;
	
	public function __construct()
	{
		$this->name = 'puntuarte';
		$this->version = '1.1.1';
		$this->author = 'Puntuarte';

		parent::__construct();
		 
		$this->displayName = $this->l('Puntuarte - Testimonials and Social Ratings');
		$this->description = $this->l('Display functional widgets with customer reviews and ratings automatically. Track your customer interaction.');

		$this->_puntuarte_module_path = _PS_MODULE_DIR_.$this->name;
		if (!Configuration::get('puntuarte_app_key'))
			$this->warning = $this->l('Set your Puntuarte API key to display your customer reviews and send new comments requests.');	

		if (!defined('_PS_BASE_URL_'))
			define('_PS_BASE_URL_', 'http://'.(isset($_SERVER['HTTP_X_FORWARDED_HOST']) ? $_SERVER['HTTP_X_FORWARDED_HOST'] : $_SERVER['HTTP_HOST']));
	}

	public function install()
	{
		if (!function_exists('curl_init'))
			$this->setError($this->l('Puntuarte needsthe PHP Curl extension, please ask your hosting provider to enable it prior to install this module.'));

		if(version_compare(_PS_VERSION_, '1.4') < 0)
			$this->setError($this->l('Minimum version required for Puntuarte module is Prestashop 1.4'));

		foreach ($this->http_request_file as $file)
			if(!file_exists($this->_puntuarte_module_path .$file))
				$this->setError($this->l('Can\'t include this file: '.$this->_puntuarte_module_path .$file));

		if ((is_array($this->_errors) && count($this->_errors) > 0) || parent::install() == false 	||
			!$this->registerHook('productfooter') 	|| !$this->registerHook('actionOrderStatusPostUpdate') ||
			!$this->registerHook('extraLeft') 		|| !$this->registerHook('extraRight') 			||
			!$this->registerHook('productTab') 		|| !$this->registerHook('productTabContent') 	|| 
			!$this->registerHook('header')) 
			return false;

		Configuration::updateValue('puntuarte_widget_location', 'footer');
		Configuration::updateValue('puntuarte_widget_tab_name', 'Reviews');
		Configuration::updateValue('puntuarte_widget_tab_name', 'productfull');
		return true;
	}

	public function hookheader()
	{
		$app_key = Configuration::get('puntuarte_app_key');
		if(isset($app_key) && !empty($app_key)) {
			$smarty = $this->context->smarty;
			$smarty->assign(array('puntuarteAppkey' => $app_key, 
								  'puntuarteDomain' => $this->getShopDomain()));
			
			return $this->display(__FILE__, 'views/templates/front/header.tpl');
		}
	}

	public function hookproductfooter($params)
	{
		$app_key = Configuration::get('puntuarte_app_key');
		$widget = Configuration::get('puntuarte_widget_name');
		if(isset($app_key) && !empty($app_key)) {
			$widgetLocation = Configuration::get('puntuarte_widget_location');
			if (Configuration::get('puntuarte_widget_location') == 'footer')
				return $this->showWidget($params['product'], $widget);
		}
	}

	public function hookActionOrderStatusPostUpdate($params)
	{	
		if ($params['newOrderStatus']->id == 5 && $params['newOrderStatus']->paid == 1 ) //Delivered && Paid
		{
			$data = $this->prepareMapData($params);
			$shop_name = Configuration::get('PS_SHOP_EMAIL');
			if (Configuration::get('puntuarte_app_key') != '' && !is_null($data)) {
				foreach ($data['products'] as $product) 
				{
				$query = 'customer_uid=' . Configuration::get('puntuarte_app_key') . '&product_sku=' . $product['reference'] . '&product_name=' . urlencode($product['name']) . 
				'&product_url=' . urlencode($product['url']) . '&to=' . $data['email'] . '&from=' . $shop_name;
				$result = $this->httpClient()->makePostRequest('https://www.puntuarte.com/API/send/mail/request_rating?' . $query, array(), 3, $parse_result = true);
				}
				return $data;
			}
		}
	}

	public function hookProductTab()
	{
		$app_key = Configuration::get('puntuarte_app_key');
		if(isset($app_key) && !empty($app_key)) {
			if ($this->parseProductId() != null && Configuration::get('puntuarte_widget_location') == 'tab') {
				$smarty = $this->context->smarty;
				$smarty->assign(array('puntuarteWidgetTabName' => Configuration::get('puntuarte_widget_tab_name')));
				return $this->display(__FILE__, 'views/templates/front/productTab.tpl');
			}		
		}
	}

	public function hookProductTabContent()
	{
		$app_key = Configuration::get('puntuarte_app_key');
		if(isset($app_key) && !empty($app_key)) {
			$product = $this->getPageProduct(null);
			if ($product != null && Configuration::get('puntuarte_widget_location') == 'tab'){
				$this->assignProductVars($product);
				$smarty = $this->context->smarty;
				$smarty->assign('puntuarteContentTplPath',dirname(__FILE__).'/views/templates/front/widgetDiv2.tpl');	
				switch (Configuration::get('puntuarte_widget_name')){
					case 'global': $smarty->assign('puntuarteContentTplPath',dirname(__FILE__).'/views/templates/front/puntuarteTotals.tpl');
									break;
					case 'productfull': 
									$smarty->assign('puntuarteContentTplPath',dirname(__FILE__).'/views/templates/front/productReviewsFull.tpl');
									break;
					case 'productmin': 
									$smarty->assign('puntuarteContentTplPath',dirname(__FILE__).'/views/templates/front/productReviewsMini.tpl');
									break;
				}				 

				return $this->display(__FILE__, 'views/templates/front/productTabContent.tpl');	
			}
				
		}
	}

	public function uninstall()
	{
		Configuration::deleteByName('puntuarte_app_key');
		Configuration::deleteByName('puntuarte_widget_location');
		Configuration::deleteByName('puntuarte_widget_tab_name');
		Configuration::deleteByName('puntuarte_widget_name');
    	Configuration::deleteByName('puntuarte_map_status');  	
		return parent::uninstall();
	}
	
	public function getContent()
	{
		if (isset($this->context->controller)){
			$this->context->controller->addCSS($this->_path.'/css/form.css', 'all');		
			$this->context->controller->addCSS('https://maxcdn.bootstrapcdn.com/bootstrap/3.3.5/css/bootstrap.min.css', 'all');
			$this->context->controller->addCSS('https://maxcdn.bootstrapcdn.com/bootstrap/3.3.5/css/bootstrap-theme.min.css', 'all');
		}
		else{
			echo '<link rel="stylesheet" type="text/css" href="../modules/puntuarte/css/form.css" />';
			echo '<link rel="stylesheet" href="https://maxcdn.bootstrapcdn.com/bootstrap/3.3.5/css/bootstrap.min.css">';
			echo '<link rel="stylesheet" href="https://maxcdn.bootstrapcdn.com/bootstrap/3.3.5/css/bootstrap-theme.min.css">';	
		}

		$this->processSettingsForm();
		return $this->displayForm();
	}

	private function getProductLink($product_id)
	{
		$link = $this->context->link;
		if (isset($link) && method_exists($link, 'getProductLink'))
			return $link->getProductLink((int)$product_id);
		else
		{
			$link = new Link();
			return $link->getProductLink((int)$product_id);	
		}
	}

	private function setError($error)
	{
		if (!$this->_errors)
			$this->_errors = array();
		$this->_errors[] = $error;
	}

	private function parseProductId()
	{
		$product_id = (int)Tools::getValue('id_product');

		if (!empty($product_id))
			return (int)$product_id;
		else
		{
			parse_str($_SERVER['QUERY_STRING'], $query);
			if (!empty($query['id_product']))
				return (int)$query['id_product'];
		}
		return null;
	}

	private function showWidget($product, $widget)
	{		
		$this->assignProductVars($product);
		switch ($widget){
			case 'global': return $this->display(__FILE__, 'views/templates/front/puntuarteTotals.tpl');
			case 'productfull': return $this->display(__FILE__, 'views/templates/front/productReviewsFull.tpl');
			case 'productmin': return $this->display(__FILE__, 'views/templates/front/productReviewsMini.tpl');
		}			
	}

	private function assignProductVars($product = null)
	{
		if(!$this->_is_smarty_product_vars_assigned)
		{
			if (is_null($product))
			$product = $this->getPageProduct();
			$this->_is_smarty_product_vars_assigned = true;
			$smarty = $this->context->smarty;
			$smarty->assign(array('puntuarteProductId' => $product->reference,
			'puntuarteProductName' => strip_tags($product->name),
			'puntuarteProductLink' => $this->getProductLink((int)$product->id)));
		}
	}
	
	private function getShopDomain()
	{
		return method_exists('Tools', 'getShopDomain') ? Tools::getShopDomain(false,false) : str_replace('www.', '', $_SERVER['HTTP_HOST']);
	}

	private function processSettingsForm()
	{
		if (Tools::isSubmit('puntuarte_settings'))
		{
			$api_key = Tools::getValue('puntuarte_app_key');
			$location = Tools::getValue('puntuarte_widget_location');
			$tabName = Tools::getValue('puntuarte_widget_tab_name');	
			$widgetName = Tools::getValue('puntuarte_widget_name');
			if ($api_key == '') return $this->prepareError($this->l('Api key is missing'));	

			$result = $this->httpClient()->makePostRequest('https://www.puntuarte.com/API/verify/api_key/' . $api_key, array(), 3, $parse_result = true);
			if($result !== 'Service API KEY validation OK'){
				return $this->prepareError("This API KEY doesn't exist, please check it in your Puntuarte Dashboard");
			}
			else{
				Configuration::updateValue('puntuarte_app_key', $api_key, false);
			}
			Configuration::updateValue('puntuarte_widget_location', $location, false);
			Configuration::updateValue('puntuarte_widget_tab_name', $tabName, false);
			Configuration::updateValue('puntuarte_widget_name', $widgetName, false);
			$this->displayConfirmation("Settings updated");
			return $this->prepareSuccess();
		}
	}

	private function displayForm($force_settings = false)
	{
		$smarty = $this->context->smarty;
		return $this->displaySettingsForm();
	}

	private function displaySettingsForm()
	{

		$smarty = $this->context->smarty;
		$smarty->assign(array(
		'puntuarte_action' => $_SERVER['REQUEST_URI'],
		'puntuarte_appKey' => Tools::getValue('puntuarte_app_key',Configuration::get('puntuarte_app_key')),
		'puntuarte_widgetName' => Configuration::get('puntuarte_widget_name'),
		'puntuarte_widgetLocation' => Configuration::get('puntuarte_widget_location'),        
		'puntuarte_tabName' => Configuration::get('puntuarte_widget_tab_name')));
		
		$settings_template = $this->display(__FILE__, 'views/templates/admin/settingsForm.tpl');
		return $this->_html .= $settings_template;
	}

	private function prepareError($message = '')
	{
		$this->_html .= sprintf('<div class="alert alert-danger fade in"><a href="#" class="close" data-dismiss="alert">&times;</a><strong>Please check your settings. </strong>%s.</div>', $message == '' ? $this->l('Error occured') : $message);
	}

	private function prepareSuccess($message = '')
	{
		$this->_html .= sprintf('<div class="alert alert-success fade in"><a href="#" class="close" data-dismiss="alert">&times;</a><strong>Success! </strong>%s.</div>', $message == '' ? $this->l('Settings updated') : $message);
	}

	private function prepareMapData($params)
	{
		$order = new Order((int)$params['id_order']);
		$customer = new Customer((int)$order->id_customer);
		$id_lang = !is_null($params['cookie']) && !is_null($params['cookie']->id_lang) ? (int)$params['cookie']->id_lang : (int)Configuration::get('PS_LANG_DEFAULT');
		if (Validate::isLoadedObject($order) && Validate::isLoadedObject($customer))
		{
			$singleMapParams = array('id_order' => (int)$params['id_order'], 'date_add' => $order->date_add,
			'email' => $customer->email, 'firstname'=> $customer->firstname, 'lastname' => $customer->lastname);

			$result = $this->getSingleMapData($singleMapParams);
			if (!is_null($result) && is_array($result))
			{
				$result['platform'] = 'prestashop';
				return $result;
			}
		}
	 	return null;
	}

	private function getSingleMapData($params)
	{
		$cart = Cart::getCartByOrderId((int)$params['id_order']);
		if(Validate::isLoadedObject($cart))
		{
			$products = $cart->getProducts();
			if(count($products) == 0 && method_exists('Shop','getContextShopID') && Shop::getContextShopID() != (int)$cart->id_shop) 
			{
				Shop::initialize();
				$products = $cart->getProducts(true);
			}					
			$currency = Currency::getCurrencyInstance((int)$cart->id_currency);
			if (!is_null($products) && is_array($products) && Validate::isLoadedObject($currency))
			{
				$data = array();
				$data['order_date'] = $params['date_add'];
				$data['email'] = $params['email'];
				$data['customer_name'] = $params['firstname'].' '.$params['lastname'];
				$products_arr = array();
				foreach ($products as $product) 
				{
					$product_data = array();    
					$product_data['url'] = $this->getProductLink($product['id_product']); 
					$product_data['name'] = $product['name'];
					$product_data['reference'] = $product['reference'];
					$products_arr[(int)$product['id_product']] = $product_data;
				}
				$data['products'] = $products_arr;
				return $data;
			}
		}
	 	return null;
	}	

	private function getPageProduct($product_id = null)
	{
		if($product_id == null)
			$product_id = $this->parseProductId();
			
		$product = new Product((int)($product_id), false, Configuration::get('PS_LANG_DEFAULT'));
		if(Validate::isLoadedObject($product))
			return $product;
			
		return null;
	}

	private function httpClient()
	{
		if (is_null($this->_httpClient))
		{
			include_once($this->_puntuarte_module_path.'/PuntuarteHttpClient.php');
			$this->_httpClient = new PuntuarteHttpClient($this->name);
		}
		return $this->_httpClient;
	}
	
}
